function I_recon = asure(I_corr, T, nPhotons)
% Image denoising of Poisson-noise corrupted image using SURE-LET approach
% on Anscombe-transformed image
% Kao Kitichotkul, 2020
%
% SURE-LET approach is based on Blu and Luisier 2007,
% "The SURE-LET Approach to Image Denoising"
%
% The details on the use of Anscombe transform for image denoising
% is drawn from Thanh and Thanh 2019, "Medical Images Denoising Method 
% Based on Total Variation Regularization and Anscombe Transform"
%
% Input
% -----
% I_corr:       Poisson-noise corrupted image, not scaled by the 
%               number of photons (pixel values in [0, 1] scale)
% T:            threshold values for soft threshold        
% nPhotons:     average number of photons on each pixel 
%               (variance of Poisson noise)   
%
% Output
% ------
% I_recon:      reconstructed image

% 
% Anscombe transform function
Ansc_fun = @(x) 2 * sqrt(x + 3/8);

% Inverse Anscombe transform function
IAnsc_fun = @(x) (x / 2).^2 - 3/8;

% Anscombe transform on the "discrete image" (scale [0, nPhotons])
I_Ansc = Ansc_fun(I_corr * nPhotons);

% Corresponding Gaussian noise level of Anscombe-transformed image
% according to Thanh and Thanh, 2019
sigma = 1;

% Soft-Threshold SURE-LET on Anscombe-transformed image on the "discrete image"
I_soft = softsure(I_Ansc, T, sigma);

% Inverse Anscombe transform and scale down to [0, 1]
I_recon = IAnsc_fun(I_soft) / nPhotons;

end

