function noise_sigma = noise_estimate(I_corr, d)
% Estimate standard deviation of Gaussian noise on an image
% Kao Kitichotkul, 2020
% 
% The algorithm is based on Chem, Zhu, and Heng 2015
% "An Efficient Statistical Method for Image Noise Level Estimation"
%
% Input
% -----
% I_corr:   noisy image
% d:        width and height of a square patch (hyperparameter). 
%           Chem, Zhu, and Heng recommended using d = 8.
%
% Output
% ------
% noise_sigma: estimated standard deviation

imSize = size(I_corr);

%%% Generate dataset

% Paramters
patchSize = d * d;
numShiftY = imSize(1) - d + 1;      
numShiftX = imSize(2) - d + 1;
numPatches = numShiftY * numShiftX;     % Number of patches

% Dataset
dataset = zeros(d, d, numPatches);
for ky = 1 : numShiftY
    for kx = 1 : numShiftX
        dataset(:, :, (ky - 1) * numShiftX + kx) = I_corr(ky : ky + d - 1, kx : kx + d - 1);
    end
end

dataset = reshape(dataset, [], numPatches);

%%% Estimate standard deviation of the noise

% Mean across samples
mu = sum(dataset, 2);

% Covariance matrix
del = dataset - mu;
Sigma = del * del'; 
Sigma = Sigma ./ numPatches;

% Eigenvalues, in descending order
eigvals = sort(eig(Sigma), 'descend');

noise_sigma = 999;
for i = 1 : patchSize
    eigvals_subset = eigvals(i:patchSize);
    tau = (1 / (patchSize - i + 1)) * sum(eigvals_subset);
    if tau == median(eigvals_subset)
        noise_sigma = abs(sqrt(tau));
        break;
    end
end

if (noise_sigma == 999)
    disp('error in noise estimation')
end

end

