function I_recon = softpure(I_corr, T, nPhotons)
% PURE-LET denoising algorithm based on soft-thresholding in DCT domain
% for Poisson-noise corrupted greyscale image
% Kao Kitichotkul, 2020
% 
% PURE-LET approach is based on Li, Luisier, and Blu 2018
% "PURE-LET Image Deconvolution"
%
% Input
% -----
% I_corr:       Poisson-noise corrupted image, not scaled by the 
%               number of photons (pixel values in [0, 1] scale)
% T:            threshold values for soft threshold        
% nPhotons:     average number of photons on each pixel 
%               (variance of Poisson noise)   
%
% Output
% ------
% I_recon:      reconstructed image

imSize = size(I_corr);
transformSize = imSize;

% noise strength parameter
alpha = 1 / nPhotons;

% Number of denoising modes
K = size(T, 2);

% Soft threshold function
soft_fun = @(w, kappa) max(1 - kappa./abs(w), 0) .* w;

%%% PURE-LET procedure

% Denoising results from each mode
F = zeros(imSize(1), imSize(2), K);                         % Denoised images
delF = zeros(transformSize(1), transformSize(2), K);        % Divergence of transformed image
for i = 1 : K
    theta_D_y = soft_fun(dct2(I_corr), T(i));
    F(:, :, i) = idct2(theta_D_y);
    delF(:, :, i) = double(theta_D_y ~= 0);
end

% Calculate a from Ma = c
M = zeros(K);
c = zeros(K, 1);
for i = 1 : K
    c(i) = sum(I_corr .* (F(:, :, i) - alpha * delF(:, :, i)), 'all');
    for j = 1 : K
        M(i, j) = sum(F(:, :, i) .* F(:, :, j), 'all');
    end
end

% Regularization parameter, as suggested by the Li, Luisier and Blu 2018
mu = 5e-3 * (mean(I_corr, 'all') ^ 2);

a_opt = (M + mu) \ c;

% Reconstruct the image
I_recon = zeros(imSize);
for i = 1 : K
    I_recon = I_recon + a_opt(i) * F(:, :, i);
end

end

