function I_recon = softsure(I_corr, T, sigma)
% SURE-LET denoising algorithm based on soft-thresholding in DCT domain.
% for Gaussian-noise corrupted greyscale image
% Kao Kitichotkul, 2020
% 
% SURE-LET approach is based on Blu and Luisier 2007,
% "The SURE-LET Approach to Image Denoising"
%
% Input
% -----
% I_corr:   noisy image
% T:        threshold values for soft threshold        
% sigma:    standard deviation of the noise         
% 
% Output
% ------
% I_recon:  reconstructed image

imSize = size(I_corr);
transformSize = imSize;

% Number of denoising modes
K = size(T, 2);

% Soft threshold function
soft_fun = @(w, kappa) max(1 - kappa./abs(w), 0) .* w;

%%% Precompute alpha = diag{DR} with the numerical method
alphaIter = 1000;
alpha = zeros(transformSize(1), transformSize(2));
for iter = 1:alphaIter
    b = randn(transformSize);
    b_prime = dct2(idct2(b));
    alpha(:, :) = alpha(:, :) + b .* b_prime;
end
alpha = alpha / alphaIter;

%%% SURE-LET procedure
% Denoising results from each mode
F = zeros(imSize(1), imSize(2), K);                         % Denoised images
theta_p = zeros(transformSize(1), transformSize(2), K);     % Divergence of transformed image
for i = 1 : K
    theta_D_y = soft_fun(dct2(I_corr), T(i));
    F(:, :, i) = idct2(theta_D_y);
    theta_p(:, :, i) = double(theta_D_y ~= 0);
end

% Calculate a from Ma = c
M = zeros(K);
c = zeros(K, 1);
for i = 1 : K
    c(i) = sum(F(:, :, i) .* I_corr, 'all') - sigma^2 * sum(alpha .* theta_p(:, :, i), 'all');
    for j = 1 : K
        M(i, j) = sum(F(:, :, i) .* F(:, :, j), 'all');
    end
end

a_opt = M \ c;

% Reconstruct the image
I_recon = zeros(imSize);
for i = 1 : K
    I_recon = I_recon + a_opt(i) * F(:, :, i);
end

end

