% Perform our denoising algorithms on images with unknown noise parameters

imageName = 'bpae1';
I = im2double(imread('images/bpae1_gt.png'));     % ground truth image
I_corr = im2double(imread('images/bpae1.png'));   % noisy image
% I = im2double(rgb2gray(imread('images/scrabble_gt.png')));     % ground truth image
% I_corr = im2double(rgb2gray(imread('images/scrabble.png')));   % noisy image
imSize = size(I);

%% Assume that the noise is Gaussian 

numMethods = 4;                                   % number of denoising methods
% Denoising parameters
T = [0 0.1 0.2 0.5];        % Threshold values for softsure
dd = 8;                     % patch size for Gaussian noise estimation
lambda = 0.05;              % lambda parameter for ADMM
rho = 10;                   % rho parameter for ADMM
numIterADMM = 100;          % number of iterations for ADMM

imResults = zeros(imSize(1), imSize(2), numMethods);
PSNR_results = zeros(numMethods, 1);

%%% Apply denoising algorithms %%%

% Method 1: Do nothing (Noisy image)
imResults(:, :, 1) = I_corr;
PSNR_results(1) = getPSNR(imResults(:, :, 1), I);
disp(['Noisy image, PSNR = ' num2str(PSNR_results(1))])
imwrite(imResults(:, :, 1), [imageName '_noisy.png'])

% Method 2: SURE-LET with Soft Thresholds
sigma_est = noise_estimate(I_corr, dd);
imResults(:, :, 2) = softsure(I_corr, T, sigma_est);
PSNR_results(2) = getPSNR(imResults(:, :, 2), I);
disp(['softsure, PSNR = ' num2str(PSNR_results(2))])
imwrite(imResults(:, :, 2), [imageName '_softsure.png'])

% Method 3: ADMM denoising with Total Variation (TV) prior
imResults(:, :, 3) = admmtv(I_corr, lambda, rho, numIterADMM);
PSNR_results(3) = getPSNR(imResults(:, :, 3), I);
disp(['admmtv, PSNR = ' num2str(PSNR_results(3))])
imwrite(imResults(:, :, 3), [imageName '_admmtv.png'])

% Method 4: ADMM denoising with Non-Local Means (NLM) prior
imResults(:, :, 4) = admmnlm(I_corr, lambda, rho, numIterADMM);
PSNR_results(4) = getPSNR(imResults(:, :, 4), I);
disp(['admmnlm, PSNR = ' num2str(PSNR_results(4))])
imwrite(imResults(:, :, 3), [imageName '_admmnlm.png'])

%% Assume that the noise is Poisson

numMethods = 3;             % number of denoising methods

% Try a range of nPhotons
nPhoton_levels = [50];

% Denoising parameters
T = [0 0.1 0.2 0.5];        % Threshold values for softsure
numIterRL = 10;             % number of iterations for Richardson-Lucy algorithm
lambdaRL = 0.02;            % lambda parameter for rltv

imResults_P = zeros(imSize(1), imSize(2), numMethods, size(nPhoton_levels, 2));
PSNR_results_P = zeros(numMethods, size(nPhoton_levels, 2));

for i = 1:size(nPhoton_levels, 2)
    
    nPhotons = nPhoton_levels(i);
    
    % Method 1: Do nothing (Noisy image)
    imResults_P(:, :, 1, i) = I_corr;
    PSNR_results_P(1, i) = getPSNR(imResults_P(:, :, 1, i), I);
    disp(['Noisy image, PSNR = ' num2str(PSNR_results_P(1, i))])
    imwrite(imResults_P(:, :, 1, i), [imageName '_noisy_nPhotons' num2str(nPhotons) '.png'])
    
    % Method 2: PURE-LET with Soft Thresholds
    imResults_P(:, :, 2, i) = softpure(I_corr, T, nPhotons);
    PSNR_results_P(2, i) = getPSNR(imResults_P(:, :, 2, i), I);
    disp(['softpure, PSNR = ' num2str(PSNR_results_P(2, i))])
    imwrite(imResults_P(:, :, 2, i), [imageName '_softpure_nPhotons' num2str(nPhotons) '.png'])
    
    % Method 3: SURE-LET on Anscombe Transformed Image
    imResults_P(:, :, 3, i) = asure(I_corr, T, nPhotons);
    PSNR_results_P(3, i) = getPSNR(imResults_P(:, :, 3, i), I);
    disp(['asure, PSNR = ' num2str(PSNR_results_P(3, i))])
    imwrite(imResults_P(:, :, 3, i), [imageName '_asure_nPhotons' num2str(nPhotons) '.png'])
    
     % Method 4: Richardson-Lucy algorithm
    imResults_P(:, :, 4, i) = rl(I_corr, nPhotons, numIterRL);
    PSNR_results_P(4, i) = getPSNR(imResults_P(:, :, 4, i), I);
    disp(['rl, PSNR = ' num2str(PSNR_results_P(4, i))])
    imwrite(imResults_P(:, :, 4, i), [imageName '_rl_nPhotons' num2str(nPhotons) '.png'])
    
     % Method 5: Richardson-Lucy algorithm with TV prior
    imResults_P(:, :, 5, i) = rltv(I_corr, nPhotons, lambdaRL, numIterRL);
    PSNR_results_P(5, i) = getPSNR(imResults_P(:, :, 5, i), I);
    disp(['rltv, PSNR = ' num2str(PSNR_results_P(5, i))])
    imwrite(imResults_P(:, :, 5, i), [imageName '_rltv_nPhotons' num2str(nPhotons) '.png'])
    
end



