% Evaluate the performance of image denoising algorithms by denoising
% synthesized noisy images.

%% Gaussian Denoising
% Apply denoising algorithms on images corrupted by additive Gaussian noise.
% The algorithms include ...
% - SURE-LET with Soft Thresholds (softsure)
% - ADMM denoising with Total Variation (TV) prior (admmtv)
% - ADMM denoising with Non-Local Means (NLM) prior (admmnlm)

imageName = 'cameraman';
I = im2double(imread('images/cameraman.tif'));   % ground truth image
% I = im2double(imread('images/goldhill.tif'));
imSize = size(I);                      % image size (vertical x horizontal)
noise_levels = [0.001 0.01 0.1];       % standard deviation of noise
numMethods = 4;                        % number of denoising methods

% Denoising parameters
T = [0 0.1 0.2 0.5];        % Threshold values for softsure
dd = 8;                     % patch size for Gaussian noise estimation
lambda = 0.05;              % lambda parameter for ADMM
rho = 10;                   % rho parameter for ADMM
numIterADMM = 100;          % number of iterations for ADMM

imResults = zeros(imSize(1), imSize(2), numMethods, size(noise_levels, 2));
PSNR_results = zeros(numMethods, size(noise_levels, 2));

for i = 1:size(noise_levels, 2)
    % Generate a Gaussian-noise corrupted sample
    sigma = noise_levels(i);
    I_corr = I + sigma * randn(imSize);
    disp(['Standard deviation of Gaussian noise \sigma = ' num2str(sigma)])
    
    % Method 1: Do nothing (Noisy image)
    imResults(:, :, 1, i) = I_corr;
    PSNR_results(1, i) = getPSNR(imResults(:, :, 1, i), I);
    disp(['Noisy image, PSNR = ' num2str(PSNR_results(1, i))])
    imwrite(imResults(:, :, 1, i), [imageName '_noisy_sig' num2str(sigma) '.png'])
    
    % Method 2: SURE-LET with Soft Thresholds
    sigma_est = noise_estimate(I_corr, dd);
    imResults(:, :, 2, i) = softsure(I_corr, T, sigma_est);
    PSNR_results(2, i) = getPSNR(imResults(:, :, 2, i), I);
    disp(['softsure, PSNR = ' num2str(PSNR_results(2, i))])
    imwrite(imResults(:, :, 2, i), [imageName '_softsure_sig' num2str(sigma) '.png'])
    
    % Method 3: ADMM denoising with Total Variation (TV) prior
    imResults(:, :, 3, i) = admmtv(I_corr, lambda, rho, numIterADMM);
    PSNR_results(3, i) = getPSNR(imResults(:, :, 3, i), I);
    disp(['admmtv, PSNR = ' num2str(PSNR_results(3, i))])
    imwrite(imResults(:, :, 3, i), [imageName '_admmtv_sig' num2str(sigma) '.png'])
    
    % Method 4: ADMM denoising with Non-Local Means (NLM) prior
    imResults(:, :, 4, i) = admmnlm(I_corr, lambda, rho, numIterADMM);
    PSNR_results(4, i) = getPSNR(imResults(:, :, 4, i), I);
    disp(['admmnlm, PSNR = ' num2str(PSNR_results(4, i))])
    imwrite(imResults(:, :, 4, i), [imageName '_admmnlm_sig' num2str(sigma) '.png'])
end


%% Poisson Denoising
% Apply denoising algorithms on images corrupted by Poisson noise.
% The algorithms include ...
% - PURE-LET with Soft Thresholds (softpure)
% - SURE-LET on Anscombe Transformed Image (asure)
% - Richardson-Lucy algorithm (rl)
% - Richardson-Lucy algorithm with TV prior (rltv)

imageName = 'cameraman';
I = im2double(imread('images/cameraman.tif'));   % ground truth image
% I = im2double(imread('images/goldhill.tif'));
imSize = size(I);                        % image size (vertical x horizontal)
nPhoton_levels = [100 500 1000];         % standard deviation of noise
numMethods = 5;                          % number of denoising methods

% Denoising parameters
T = [0 0.1 0.2 0.5];        % Threshold values for softsure
numIterRL = 10;             % number of iterations for Richardson-Lucy algorithm
lambdaRL = 0.02;            % lambda parameter for rltv

imResults_P = zeros(imSize(1), imSize(2), numMethods, size(nPhoton_levels, 2));
PSNR_results_P = zeros(numMethods, size(nPhoton_levels, 2));


for i = 1:size(nPhoton_levels, 2)
    % Generate a Poisson-noise corrupted sample
    nPhotons = nPhoton_levels(i);
    I_corr = poissrnd(I .* nPhotons) / nPhotons;
    disp(['Poisson noise, number of photons = ' num2str(nPhotons)])
    
    % Method 1: Do nothing (Noisy image)
    imResults_P(:, :, 1, i) = I_corr;
    PSNR_results_P(1, i) = getPSNR(imResults_P(:, :, 1, i), I);
    disp(['Noisy image, PSNR = ' num2str(PSNR_results_P(1, i))])
    imwrite(imResults_P(:, :, 1, i), [imageName '_noisy_nPhotons' num2str(nPhotons) '.png'])
    
    % Method 2: PURE-LET with Soft Thresholds
    imResults_P(:, :, 2, i) = softpure(I_corr, T, nPhotons);
    PSNR_results_P(2, i) = getPSNR(imResults_P(:, :, 2, i), I);
    disp(['softpure, PSNR = ' num2str(PSNR_results_P(2, i))])
    imwrite(imResults_P(:, :, 2, i), [imageName '_softpure_nPhotons' num2str(nPhotons) '.png'])
    
    % Method 3: SURE-LET on Anscombe Transformed Image
    imResults_P(:, :, 3, i) = asure(I_corr, T, nPhotons);
    PSNR_results_P(3, i) = getPSNR(imResults_P(:, :, 3, i), I);
    disp(['asure, PSNR = ' num2str(PSNR_results_P(3, i))])
    imwrite(imResults_P(:, :, 3, i), [imageName '_asure_nPhotons' num2str(nPhotons) '.png'])
    
    % Method 4: Richardson-Lucy algorithm
    imResults_P(:, :, 4, i) = rl(I_corr, nPhotons, numIterRL);
    PSNR_results_P(4, i) = getPSNR(imResults_P(:, :, 4, i), I);
    disp(['rl, PSNR = ' num2str(PSNR_results_P(4, i))])
    imwrite(imResults_P(:, :, 4, i), [imageName '_rl_nPhotons' num2str(nPhotons) '.png'])
    
    % Method 5: Richardson-Lucy algorithm with TV prior
    imResults_P(:, :, 5, i) = rltv(I_corr, nPhotons, lambdaRL, numIterRL);
    PSNR_results_P(5, i) = getPSNR(imResults_P(:, :, 5, i), I);
    disp(['rltv, PSNR = ' num2str(PSNR_results_P(5, i))])
    imwrite(imResults_P(:, :, 5, i), [imageName '_rltv_nPhotons' num2str(nPhotons) '.png'])
end



